@@ -86,13 +86,13 @@ smle_select<-function(object, ...){
 #'
 #' @param gamma_ebic The EBIC tuning parameter, in \eqn{[0 , 1]}. Default is 0.5.
 #'
-#' @param vote The logical flag for whether to perform the voting procedure. Only available when \code{criterion ="ebic"}.
+#' @param vote The logical flag for whether to perform the voting procedure. Only available when \code{criterion = "ebic"}.
 #' 
 #' @param criterion Selection criterion. One of "\code{ebic}","\code{bic}","\code{aic}". Default is "\code{ebic}".
 #'
 #' @param codingtype Coding types for categorical features; for more details see \code{\link{SMLE}()} documentation.
 #'
-#' @param gamma_seq The sequence of values for \code{gamma_ebic} when \code{vote =TRUE}.
+#' @param gamma_seq The sequence of values for \code{gamma_ebic} when \code{vote = TRUE}.
 #'
 #' @param vote_threshold A relative voting threshold in percentage. A feature is
 #'  considered to be important when it receives votes passing the threshold. Default is 0.6.
@@ -100,21 +100,28 @@ smle_select<-function(object, ...){
 #' @param parallel A logical flag to use parallel computing to do voting selection.
 #' Default is \code{FALSE}. See Details.
 #'
+#' @param keyset A numeric vector with column indices for the key features that 
+#' do not participate in feature screening and are forced to remain in the model. See SMLE for details.
 #' @param num_clusters The number of compute clusters to use when 
 #' \code{parallel = TRUE}. The default will be 2 times cores detected.
 #'
 #' @export
 #'
 smle_select.sdata<-function(object, k_min=1, k_max=NULL, subset=NULL,
-                            gamma_ebic=0.5, vote= FALSE,
-                            criterion="ebic", codingtype = NULL,
-                            gamma_seq=c(seq(0,1,0.2)), vote_threshold=NULL,
+                            gamma_ebic=0.5, vote= FALSE, keyset=NULL,
+                            criterion="ebic", codingtype = c("DV","standard","all"),
+                            gamma_seq=c(seq(0,1,0.2)), vote_threshold=0.6,
                             parallel = FALSE, num_clusters=NULL,...){
   cl<-match.call()
   
+  codingtype <- match.arg( codingtype )
+  
   cl[[1]] <- as.name("smle_select")
   
-  #input check
+  #----------------------------------------------------------------------------#
+  #------------------------- Argument pre-processing --------------------------#
+  #----------------------------------------------------------------------------#
+  
   X<-object$X
 
   Y<-object$Y
@@ -125,116 +132,118 @@ smle_select.sdata<-function(object, k_min=1, k_max=NULL, subset=NULL,
 
   pp<- dim(X)[2]
   
-
-  #check input data dimension, warning for High dimensional input.
+  # Check input data dimension, warning for High dimensional input.
   if(is.null(subset)){
 
-    if(pp>1000 & pp>n ){
-      message("For high dimensional data input, we recommand to do screening before selection")
+    if(pp>999){
+      
+      message("For high dimensional data input, we recommand to do screening before selection. ")
+      Continue <- readline(prompt="Do you want to proceed? Y or N \n ")
+      
+      while(T){
+        
+        if(Continue == "N"){
+          
+          tryCatch(stop(), error = function(e) message('Algorithm aborted'))
+          
+        }else if(Continue == "Y"){
+          
+          message('Algorithm processing')
+          
+          break
+          
+        }else{
+          
+          Continue <- readline(prompt='Please input Y or N. \n ')
+        }
+        
+        
+      }
+      
+      
     }
+    
     X_s<-X
 
     }else{
 
       X_s<-X[,subset]
 
-
-      }
-
-  k_max = dim(X_s)[2]
+    }
   
-  if ( is.null(vote_threshold) ){
-
-    vote_threshold=0.6
-
+  if(!is.null(keyset)){
+    
+    keyset <- which(subset %in% keyset)
+    
+    k_min <- max(k_min, length(keyset)+1)
+    
+    }
+  
+  
+  categorical <- any(sapply(X_s,is.factor))
+  
+  
+  if(is.null(k_max)){
+    
+    k_max = dim(X_s)[2]
+    
   }
-  if(parallel==TRUE && is.null(num_clusters)){ 
+
+  if(parallel && is.null(num_clusters)){ 
     
     num_cores    <- parallel::detectCores() 
+    
     num_clusters <- 2 * num_cores
-    }
-
-  IP<-NULL;ID_Voted<-NULL;vs<-NULL
-
-  if(k_min<0 || k_max<k_min ||k_min%%1!=0 ||k_max%%1 !=0||k_max>pp){
-
-    stop("Retained model size setting error(k_min and k_max).")
-
-    }
-
-  #Feature selection by criterion.
-
-  if(is.null( codingtype )){codingtype<-"DV"}
-
-  if(!codingtype%in% c("all","standard","DV")){stop("Codingtype only in
-                                                    all,standard,DV")}
-
-  #-----------------------------Algorithm-start---------------------------------
-  categorical =FALSE
-  
-  CI = (1:dim(X_s)[2])[sapply(X_s,is.factor)]
-  
-  if( any(  CI )){
     
-    #if sub matrix X_s has any categorical features
-    
-    categorical= TRUE
-    
-    criter_value<-ctg_ebicc(Y,X_s,family,criterion,codingtype,
-                            k_min,k_max,n,pp,gamma_ebic,parallel,num_clusters)
-
-    v_s <- which.min(criter_value)+k_min-1
-
-    f_s<- SMLE(Y=Y, X=X_s, k=v_s, family=family,categorical = T)
-
-    #Feature selection by ebic voting.
-
-    ebic_selection_by_gamma<-function(gamma){
-
-      ebic_value<-ctg_ebicc(Y,X_s,family,criterion,codingtype,
-                            k_min,k_max,n,pp,gamma,parallel,num_clusters)
-
-      v_s <- which.min(ebic_value)+k_min-1
-
-      ID <- SMLE(Y=Y, X=X_s, k=v_s, family=family,categorical = T)$ID_retained
-      
-      ID
-
     }
 
-  }else{
+  IP <- NULL; ID_Voted <- NULL; vs <- NULL
 
+  if(k_min<0 || k_max < k_min ||k_min%%1!=0 ||k_max%%1 !=0||k_max>pp){
 
-    criter_value<-ebicc(Y,X_s,family,criterion,k_min,k_max,n,pp,gamma_ebic,parallel,num_clusters)
-
-    v_s <- which.min(criter_value)+k_min-1
-
-    f_s<- SMLE(Y=Y, X=X_s, k=v_s, family=family)
-
-    #Feature selection by ebic voting.
-
-    ebic_selection_by_gamma<-function(gamma){
-
-      ebic_value<-ebicc(Y,X_s,family,criterion,k_min,k_max,n,pp,gamma,parallel,num_clusters)
-
-      v_s <- which.min(ebic_value)+k_min-1
-
-      ID <- SMLE(Y=Y, X=X_s, k=v_s, family=family)$ID_retained
-      
-      ID
-
-    }
-
+    tryCatch(stop(), error = function(e) message('Retained model size setting error(k_min and k_max).'))
   }
+  
+  #----------------------------------------------------------------------------#
+  #------------------------------ Selection -----------------------------------#
+  #----------------------------------------------------------------------------#
+
+  # Run selection algorithm with information criterion specified (EBIC default)
+    
+  criter_value<- ebicc(Y,X_s,family,criterion,codingtype,keyset,
+                            k_min,k_max,n,pp,gamma_ebic,parallel,num_clusters)
+    
+  v_s <- which.min(criter_value)+k_min-1
+  
+  f_s<- SMLE(Y=Y, X=X_s, k=v_s, family=family,keyset=keyset)
 
 
-  if(vote==TRUE){
+    #------------------------------------------------------------------------#
+    #------------------------------- Voting ---------------------------------#
+    #------------------------------------------------------------------------#
 
-    stopifnot(criterion == 'ebic')
+  if( vote ){
+
+    stopifnot("Voting available when criterion = ebic"=criterion == 'ebic')
 
     vs<-c()
+    
+    ebic_selection_by_gamma<-function(gamma){
+      
+      ebic_value<-ebicc(Y,X_s,family,criterion,codingtype,keyset,
+                        k_min,k_max,n,pp,gamma,parallel,num_clusters)
+      
+      v_s <- which.min(ebic_value)+k_min-1
+      
+      ID <- SMLE(Y=Y, X=X_s, k=v_s, family=family,keyset=keyset)$ID_retained
+      
+      ID
+      
+    }
+    
+    ## Run selection algorithm with different gammas 
 
-    if(parallel==TRUE){
+    if(parallel ){
       
       if(.Platform$OS.type=="windows"){
         
@@ -248,11 +257,18 @@ smle_select.sdata<-function(object, k_min=1, k_max=NULL, subset=NULL,
           
       vs<-unlist(lapply(gamma_seq,ebic_selection_by_gamma))
 
-      }
+        }
+    
+    #----------------------------------------------------------------#
+    
     IP<-as.factor(vs)
     
+    # Count the frequency of features selected with different model of sparsity.
+    
     IP_f<-summary(IP)[order(summary(IP),decreasing= T)]/max(summary(IP))
     
+    # features with frequency higher than vote_threshold will be selected in ID_voted.
+    
     ID_Voted<-as.numeric(names(IP_f[IP_f>=vote_threshold]))
    
 
@@ -269,10 +285,9 @@ smle_select.sdata<-function(object, k_min=1, k_max=NULL, subset=NULL,
 
   }
 
-  S<-list(
-          call = cl,
+  Out<-list(
     
-          ID_selected=ID_Selected, family = family,
+          call = cl, ID_selected=ID_Selected, family = family,
 
           coef_selected=f_s$coef_retained,
           
@@ -282,7 +297,7 @@ smle_select.sdata<-function(object, k_min=1, k_max=NULL, subset=NULL,
 
           vote = vote,criterion = criterion,
           
-          ID_pool= IP,
+          ID_pool= IP, keyset = keyset, k_min = k_min, k_max= k_max,
           
           subset = subset,
 
@@ -290,7 +305,7 @@ smle_select.sdata<-function(object, k_min=1, k_max=NULL, subset=NULL,
 
           ID_voted = ID_Voted,
           
-          categorical = categorical, CI= sort(CI),
+          categorical = categorical,
           
           vote_threshold=vote_threshold,
 
@@ -299,8 +314,10 @@ smle_select.sdata<-function(object, k_min=1, k_max=NULL, subset=NULL,
           codingtype = codingtype,
 
           gamma_seq=gamma_seq, X = X , Y = Y)
-  class(S)<-"selection"
-  return(S)
+  
+  class(Out)<-"selection"
+  
+  return(Out)
   }
 
 #' @rdname smle_select
@@ -320,30 +337,49 @@ smle_select.sdata<-function(object, k_min=1, k_max=NULL, subset=NULL,
 #'
 #' @export
 #'
-smle_select.default<-function(object = NULL,Y =NULL, X=NULL, family='gaussian',...){
+smle_select.default<-function(object = NULL,Y =NULL, X=NULL, family='gaussian',keyset = NULL,...){
+  
+  # If the data is passed, this function create the sdata object, and then call 
+  # smle_select with the sdata function call.
+  
   cl<-match.call()
+  
   cl[[1]] <- as.name("smle_select")
-  if ( any(sapply(X,as.factor) ==TRUE) ){
-   categorical = TRUE
-  }else{
-   categorical =FALSE
+  
+  if ( any(sapply(X,is.factor) ) ){
+  
+     categorical = TRUE
+  
+     }else{
+   
+       categorical =FALSE
+       
   }
   if(is.null(object)){
     
-    Data<-structure(list(Y=Y,X=X,family=family,categorical = categorical),class = "sdata")
+    object<-Y
   
-  }else{
-    
-    Data<-structure(list(Y=object,X=X,family=family,categorical = categorical),class = "sdata")
+  }
+  
+  if(!is.null(keyset)){
     
+    if( inherits( keyset , c('integer','numeric'))  ){
+      
+      keyset_ind <- as.character(colnames(X)[keyset])
+      
     }
+    
+    keyset_ind <- which(colnames(X) %in% keyset)
+    
+  }else{keyset_ind <- keyset}
+    
+  Data<-structure(list(Y=object,X=X,family=family,categorical = categorical),class = "sdata")
 
+  Out<-smle_select(Data, keyset = keyset_ind,...)
   
-
-
-  S<-smle_select(Data,...)
-  S$call <- cl
-  return(S)
+  Out$call <- cl
+  
+  return(Out)
 }
 #' @rdname  smle_select
 #'
@@ -351,30 +387,65 @@ smle_select.default<-function(object = NULL,Y =NULL, X=NULL, family='gaussian',.
 #'
 #' @export
 #'
+
 smle_select.smle<-function(object,...){
+  
+  # When a smle object is called, this function create the sdata object and set 
+  # the retained features ID in smle object as the argument subset in smle_select,
+  # and then call smle_select with the sdata function call.
+  
   cl<-match.call()
+  
   cl[[1]] <- as.name("smle_select")
+  
   Data<-structure(list( Y=object$Y, X=object$X, family=object$family),class = "sdata")
+  
   if(!is.null(object$data)){
+    
     data<-object$data
+    
     X <- object$X
-    index <- sort((1:length(colnames(X)))[colnames(X) %in% object$iteration_data$feature_name])
-    S<-smle_select(Data,subset =index,...)
-    feature_name <- colnames(X)[S$ID_selected]
-    S$ID_selected <-  sort((1:length(names(data)))[names(data) %in% feature_name])
-    if(S$vote == TRUE){
+    
+    keyset <- object$keyset
+    
+    if( inherits( keyset , c('integer','numeric'))  ){
       
-      vote_name<-colnames(X)[S$ID_voted]
-      S$ID_voted <-sort((1:length(names(data)))[names(data) %in% vote_name])
+      keyset <- as.character(names(data)[keyset])
+      
+    }
+    
+    index <- which(colnames(X) %in% object$iteration_data$feature_name)
+    
+    keyset_ind <- which(colnames(X) %in% keyset)
+    
+    Out<-smle_select(Data,subset =index, keyset = keyset_ind,...)
+    
+    feature_name <- colnames(X)[Out$ID_selected]
+    
+    Out$ID_selected <-  sort(which(names(data) %in% feature_name))
+    
+    if(Out$vote == TRUE){
+        
+      vote_name<-colnames(X)[Out$ID_voted]
+      
+      Out$ID_voted <-sort(which(names(data) %in% vote_name))
 
     }
     
-    S$data<- data
+    Out$data<- data
+    
+    Out$keyset <- keyset
+    
+  }else{ 
+    
+    Out<-smle_select(Data,subset =object$ID_retained,keyset = object$keyset,...)
+    
+    }
   
-  }else{    S<-smle_select(Data,subset =object$ID_retained,...)}
+  Out$call <- cl
   
-  S$call <- cl
-  return(S)
+  Out$keyset <- object$keyset
+  return(Out)
 }
 
 
